@@ -1,12 +1,14 @@
-/*********************************************************************
- * Copyright (c) 2020 Red Hat, Inc.
- *
+/**
+ * Copyright (c) 2019-2021 Red Hat, Inc.
  * This program and the accompanying materials are made
  * available under the terms of the Eclipse Public License 2.0
  * which is available at https://www.eclipse.org/legal/epl-2.0/
  *
  * SPDX-License-Identifier: EPL-2.0
- **********************************************************************/
+ *
+ * Contributors:
+ *   Red Hat, Inc. - initial API and implementation
+ */
 
 import Command from '@oclif/command'
 import { cli } from 'cli-ux'
@@ -24,7 +26,9 @@ import { createEclipseCheCluster, createNamespaceTask, patchingEclipseCheCluster
 
 export class OLMTasks {
   prometheusRoleName = 'prometheus-k8s'
+
   prometheusRoleBindingName = 'prometheus-k8s'
+
   /**
    * Returns list of tasks which perform preflight platform checks.
    */
@@ -45,7 +49,7 @@ export class OLMTasks {
             await kube.createRoleFromFile(yamlFilePath, flags.chenamespace)
             task.title = `${task.title}...done.`
           }
-        }
+        },
       },
       {
         enabled: () => flags['cluster-monitoring'] && flags.platform === 'openshift',
@@ -60,7 +64,7 @@ export class OLMTasks {
             await kube.createRoleBindingFromFile(yamlFilePath, flags.chenamespace)
             task.title = `${task.title}...done.`
           }
-        }
+        },
       },
       {
         title: 'Create operator group',
@@ -71,7 +75,7 @@ export class OLMTasks {
             await kube.createOperatorGroup(OPERATOR_GROUP_NAME, flags.chenamespace)
             task.title = `${task.title}...created new one.`
           }
-        }
+        },
       },
       {
         title: 'Configure context information',
@@ -103,10 +107,10 @@ export class OLMTasks {
           }
 
           task.title = `${task.title}...done.`
-        }
+        },
       },
       {
-        enabled: () => !VersionHelper.isDeployingStableVersion(flags) && !flags['catalog-source-name'] && !flags['catalog-source-yaml'],
+        enabled: () => !VersionHelper.isDeployingStableVersion(flags) && !flags['catalog-source-name'] && !flags['catalog-source-yaml'] && flags['olm-channel'] !== OLM_STABLE_CHANNEL_NAME,
         title: `Create nightly index CatalogSource in the namespace ${flags.chenamespace}`,
         task: async (ctx: any, task: any) => {
           if (!await kube.catalogSourceExists(NIGHTLY_CATALOG_SOURCE_NAME, flags.chenamespace)) {
@@ -117,7 +121,7 @@ export class OLMTasks {
           } else {
             task.title = `${task.title}...It already exists.`
           }
-        }
+        },
       },
       {
         enabled: () => flags['catalog-source-yaml'],
@@ -133,7 +137,7 @@ export class OLMTasks {
           } else {
             task.title = `${task.title}...It already exists.`
           }
-        }
+        },
       },
       {
         title: 'Create operator subscription',
@@ -146,7 +150,7 @@ export class OLMTasks {
               // custom Che CatalogSource
               const catalogSourceNamespace = flags['catalog-source-namespace'] || flags.chenamespace
               subscription = this.constructSubscription(SUBSCRIPTION_NAME, flags['package-manifest-name'], flags.chenamespace, catalogSourceNamespace, flags['olm-channel'], ctx.sourceName, ctx.approvalStarategy, ctx.startingCSV)
-            } else if (VersionHelper.isDeployingStableVersion(flags)) {
+            } else if (VersionHelper.isDeployingStableVersion(flags) || flags['olm-channel'] === OLM_STABLE_CHANNEL_NAME) {
               // stable Che CatalogSource
               subscription = this.constructSubscription(SUBSCRIPTION_NAME, DEFAULT_CHE_OLM_PACKAGE_NAME, flags.chenamespace, ctx.defaultCatalogSourceNamespace, OLM_STABLE_CHANNEL_NAME, ctx.catalogSourceNameStable, ctx.approvalStarategy, ctx.startingCSV)
             } else {
@@ -156,7 +160,7 @@ export class OLMTasks {
             await kube.createOperatorSubscription(subscription)
             task.title = `${task.title}...created new one.`
           }
-        }
+        },
       },
       {
         title: 'Wait while subscription is ready',
@@ -164,7 +168,7 @@ export class OLMTasks {
           const installPlan = await kube.waitOperatorSubscriptionReadyForApproval(flags.chenamespace, SUBSCRIPTION_NAME, 600)
           ctx.installPlanName = installPlan.name
           task.title = `${task.title}...done.`
-        }
+        },
       },
       {
         title: 'Approve installation',
@@ -172,14 +176,14 @@ export class OLMTasks {
         task: async (ctx: any, task: any) => {
           await kube.approveOperatorInstallationPlan(ctx.installPlanName, flags.chenamespace)
           task.title = `${task.title}...done.`
-        }
+        },
       },
       {
         title: 'Wait while operator installed',
         task: async (ctx: any, task: any) => {
           await kube.waitUntilOperatorIsInstalled(ctx.installPlanName, flags.chenamespace)
           task.title = `${task.title}...done.`
-        }
+        },
       },
       {
         title: 'Set custom operator image',
@@ -193,7 +197,7 @@ export class OLMTasks {
           const jsonPatch = [{ op: 'replace', path: '/spec/install/spec/deployments/0/spec/template/spec/containers/0/image', value: flags['che-operator-image'] }]
           await kube.patchClusterServiceVersion(csv.metadata.namespace!, csv.metadata.name!, jsonPatch)
           task.title = `${task.title}... changed to ${flags['che-operator-image']}.`
-        }
+        },
       },
       {
         title: 'Prepare CodeReady Workspaces cluster CR',
@@ -209,9 +213,9 @@ export class OLMTasks {
           }
 
           task.title = `${task.title}...Done.`
-        }
+        },
       },
-      createEclipseCheCluster(flags, kube)
+      createEclipseCheCluster(flags, kube),
     ], { renderer: flags['listr-renderer'] as any })
   }
 
@@ -226,7 +230,7 @@ export class OLMTasks {
             command.error(`Unable to find operator group ${OPERATOR_GROUP_NAME}`)
           }
           task.title = `${task.title}...done.`
-        }
+        },
       },
       {
         title: 'Check if operator subscription exists',
@@ -235,7 +239,7 @@ export class OLMTasks {
             command.error(`Unable to find operator subscription ${SUBSCRIPTION_NAME}`)
           }
           task.title = `${task.title}...done.`
-        }
+        },
       },
     ], { renderer: flags['listr-renderer'] as any })
   }
@@ -276,7 +280,7 @@ export class OLMTasks {
             }
           }
           command.error('Unable to find installation plan to update.')
-        }
+        },
       },
       {
         title: 'Approve installation',
@@ -284,18 +288,18 @@ export class OLMTasks {
         task: async (ctx: any, task: any) => {
           await kube.approveOperatorInstallationPlan(ctx.installPlanName, flags.chenamespace)
           task.title = `${task.title}...done.`
-        }
+        },
       },
       {
         title: 'Wait while newer operator installed',
         enabled: (ctx: any) => ctx.installPlanName,
         task: async (ctx: any, task: any) => {
           await kube.waitUntilOperatorIsInstalled(ctx.installPlanName, flags.chenamespace, 60)
-          ctx.highlightedMessages.push(`Operator is updated from ${ctx.currentVersion} to ${ ctx.nextVersion} version`)
+          ctx.highlightedMessages.push(`Operator is updated from ${ctx.currentVersion} to ${ctx.nextVersion} version`)
           task.title = `${task.title}...done.`
-        }
+        },
       },
-      patchingEclipseCheCluster(flags, kube, command)
+      patchingEclipseCheCluster(flags, kube, command),
     ], { renderer: flags['listr-renderer'] as any })
   }
 
@@ -305,19 +309,17 @@ export class OLMTasks {
       {
         title: 'Check if OLM is pre-installed on the platform',
         task: async (ctx: any, task: any) => {
-          ctx.isPreInstalledOLM = await kube.isPreInstalledOLM() ? true : false
+          ctx.isPreInstalledOLM = Boolean(await kube.isPreInstalledOLM())
           task.title = `${task.title}: ${ctx.isPreInstalledOLM}...OK`
-        }
+        },
       },
       {
         title: `Delete(OLM) operator subscription ${SUBSCRIPTION_NAME}`,
         enabled: ctx => ctx.isPreInstalledOLM,
         task: async (_ctx: any, task: any) => {
-          if (await kube.operatorSubscriptionExists(SUBSCRIPTION_NAME, flags.chenamespace)) {
-            await kube.deleteOperatorSubscription(SUBSCRIPTION_NAME, flags.chenamespace)
-          }
+          await kube.deleteOperatorSubscription(SUBSCRIPTION_NAME, flags.chenamespace)
           task.title = `${task.title}...OK`
-        }
+        },
       },
       {
         title: 'Delete(OLM) CodeReady Workspaces cluster service versions',
@@ -327,53 +329,43 @@ export class OLMTasks {
           const csvsToDelete = csvs.items.filter(csv => csv.metadata.name!.startsWith(CVS_PREFIX))
           csvsToDelete.forEach(csv => kube.deleteClusterServiceVersion(flags.chenamespace, csv.metadata.name!))
           task.title = `${task.title}...OK`
-        }
+        },
       },
       {
         title: `Delete(OLM) operator group ${OPERATOR_GROUP_NAME}`,
         enabled: ctx => ctx.isPreInstalledOLM,
         task: async (_ctx: any, task: any) => {
-          if (await kube.operatorGroupExists(OPERATOR_GROUP_NAME, flags.chenamespace)) {
-            await kube.deleteOperatorGroup(OPERATOR_GROUP_NAME, flags.chenamespace)
-          }
+          await kube.deleteOperatorGroup(OPERATOR_GROUP_NAME, flags.chenamespace)
           task.title = `${task.title}...OK`
-        }
+        },
       },
       {
         title: `Delete(OLM) custom catalog source ${CUSTOM_CATALOG_SOURCE_NAME}`,
         task: async (_ctx: any, task: any) => {
-          if (await kube.catalogSourceExists(CUSTOM_CATALOG_SOURCE_NAME, flags.chenamespace)) {
-            await kube.deleteCatalogSource(flags.chenamespace, CUSTOM_CATALOG_SOURCE_NAME)
-          }
+          await kube.deleteCatalogSource(flags.chenamespace, CUSTOM_CATALOG_SOURCE_NAME)
           task.title = `${task.title}...OK`
-        }
+        },
       },
       {
         title: `Delete(OLM) nigthly catalog source ${NIGHTLY_CATALOG_SOURCE_NAME}`,
         task: async (_ctx: any, task: any) => {
-          if (await kube.catalogSourceExists(NIGHTLY_CATALOG_SOURCE_NAME, flags.chenamespace)) {
-            await kube.deleteCatalogSource(flags.chenamespace, NIGHTLY_CATALOG_SOURCE_NAME)
-          }
+          await kube.deleteCatalogSource(flags.chenamespace, NIGHTLY_CATALOG_SOURCE_NAME)
           task.title = `${task.title}...OK`
-        }
+        },
       },
       {
         title: `Delete role ${this.prometheusRoleName}`,
         task: async (_ctx: any, task: any) => {
-          if (await kube.roleExist(this.prometheusRoleName, flags.chenamespace)) {
-            await kube.deleteRole(this.prometheusRoleName, flags.chenamespace)
-          }
+          await kube.deleteRole(this.prometheusRoleName, flags.chenamespace)
           task.title = await `${task.title}...OK`
-        }
+        },
       },
       {
         title: `Delete role binding ${this.prometheusRoleName}`,
         task: async (_ctx: any, task: any) => {
-          if (await kube.roleBindingExist(this.prometheusRoleName, flags.chenamespace)) {
-            await kube.deleteRoleBinding(this.prometheusRoleName, flags.chenamespace)
-          }
+          await kube.deleteRoleBinding(this.prometheusRoleName, flags.chenamespace)
           task.title = await `${task.title}...OK`
-        }
+        },
       },
     ]
   }
@@ -388,7 +380,7 @@ export class OLMTasks {
           command.error('OLM is required for installation CodeReady Workspaces with installer flag \'olm\'')
         }
         task.title = `${task.title}...done.`
-      }
+      },
     }
   }
 
@@ -398,7 +390,7 @@ export class OLMTasks {
       kind: 'Subscription',
       metadata: {
         name,
-        namespace
+        namespace,
       },
       spec: {
         channel,
@@ -407,7 +399,7 @@ export class OLMTasks {
         source: sourceName,
         sourceNamespace,
         startingCSV,
-      }
+      },
     }
   }
 
@@ -424,10 +416,10 @@ export class OLMTasks {
         sourceType: 'grpc',
         updateStrategy: {
           registryPoll: {
-            interval: '15m'
-          }
-        }
-      }
+            interval: '15m',
+          },
+        },
+      },
     }
   }
 
@@ -444,7 +436,7 @@ export class OLMTasks {
   }
 
   private getOlmNamespaceLabels(flags: any): any {
-    //The label values must be strings
+    // The label values must be strings
     if (flags['cluster-monitoring'] && flags.platform === 'openshift') {
       return { 'openshift.io/cluster-monitoring': 'true' }
     }
