@@ -1,12 +1,14 @@
-/*********************************************************************
- * Copyright (c) 2019 Red Hat, Inc.
- *
+/**
+ * Copyright (c) 2019-2021 Red Hat, Inc.
  * This program and the accompanying materials are made
  * available under the terms of the Eclipse Public License 2.0
  * which is available at https://www.eclipse.org/legal/epl-2.0/
  *
  * SPDX-License-Identifier: EPL-2.0
- **********************************************************************/
+ *
+ * Contributors:
+ *   Red Hat, Inc. - initial API and implementation
+ */
 import { V1ClusterRole, V1ClusterRoleBinding, V1Deployment, V1Role, V1RoleBinding } from '@kubernetes/client-node'
 import { Command } from '@oclif/command'
 import { cli } from 'cli-ux'
@@ -17,15 +19,21 @@ import * as path from 'path'
 import { ChectlContext } from '../../api/context'
 import { KubeHelper } from '../../api/kube'
 import { VersionHelper } from '../../api/version'
-import { CHE_CLUSTER_CRD, CHE_OPERATOR_SELECTOR, OPERATOR_DEPLOYMENT_NAME, OPERATOR_TEMPLATE_DIR } from '../../constants'
-import { safeLoadFromYamlFile } from '../../util'
+import { CHE_CLUSTER_API_GROUP, CHE_CLUSTER_API_VERSION, CHE_CLUSTER_CRD, CHE_CLUSTER_KIND_PLURAL, CHE_OPERATOR_SELECTOR, OPERATOR_DEPLOYMENT_NAME, OPERATOR_TEMPLATE_DIR } from '../../constants'
+import { getImageNameAndTag, safeLoadFromYamlFile } from '../../util'
 import { KubeTasks } from '../kube'
 
 import { createEclipseCheCluster, createNamespaceTask, patchingEclipseCheCluster } from './common-tasks'
 
 export class OperatorTasks {
   operatorServiceAccount = 'codeready-operator'
+
   cheClusterCrd = 'checlusters.org.eclipse.che'
+
+  cheClusterBackupCrd = 'checlusterbackups.org.eclipse.che'
+
+  cheClusterRestoreCrd = 'checlusterrestores.org.eclipse.che'
+
   legacyClusterResourcesName = 'codeready-operator'
 
   private getReadRolesAndBindingsTask(kube: KubeHelper): Listr.ListrTask {
@@ -73,7 +81,7 @@ export class OperatorTasks {
         }
 
         task.title = `${task.title}...done.`
-      }
+      },
     }
   }
 
@@ -137,19 +145,19 @@ export class OperatorTasks {
         }
 
         task.title = `${task.title}...done.`
-      }
+      },
     }
   }
 
   /**
    * Returns tasks list which perform preflight platform checks.
    */
-  deployTasks(flags: any, command: Command): Listr {
+  async deployTasks(flags: any, command: Command): Promise<Listr> {
     const kube = new KubeHelper(flags)
     const kubeTasks = new KubeTasks(flags)
     const ctx = ChectlContext.get()
     ctx.resourcesPath = path.join(flags.templates, OPERATOR_TEMPLATE_DIR)
-    if (VersionHelper.isDeployingStableVersion(flags)) {
+    if (VersionHelper.isDeployingStableVersion(flags) && !await kube.isOpenShift3()) {
       command.warn('Consider using the more reliable \'OLM\' installer when deploying a stable release of CodeReady Workspaces (--installer=olm).')
     }
     return new Listr([
@@ -165,34 +173,60 @@ export class OperatorTasks {
             await kube.createServiceAccountFromFile(yamlFilePath, flags.chenamespace)
             task.title = `${task.title}...done.`
           }
-        }
+        },
       },
       this.getReadRolesAndBindingsTask(kube),
       this.getCreateOrUpdateRolesAndBindingsTask(flags, 'Creating Roles and Bindings', false),
       {
         title: `Create CRD ${this.cheClusterCrd}`,
         task: async (ctx: any, task: any) => {
-          const exist = await kube.crdExist(this.cheClusterCrd)
-          const yamlFilePath = path.join(ctx.resourcesPath, 'crds', 'org_v1_che_crd.yaml')
-
-          if (exist) {
-            const checkCRD = await kube.isCRDCompatible(this.cheClusterCrd, yamlFilePath)
-            if (!checkCRD) {
-              cli.error(`It is not possible to proceed the installation of CodeReady Workspaces. The existed ${this.cheClusterCrd} is different from a new one. Please update it to continue the installation.`)
-            }
+          const existedCRD = await kube.getCrd(this.cheClusterCrd)
+          if (existedCRD) {
             task.title = `${task.title}...It already exists.`
           } else {
-            await kube.createCrdFromFile(yamlFilePath)
+            const newCRDPath = await this.getCRDPath(ctx, flags)
+            await kube.createCrdFromFile(newCRDPath)
             task.title = `${task.title}...done.`
           }
-        }
+        },
+      },
+      {
+        title: 'Create backup and restore CRDs',
+        task: async (ctx: any, task: any) => {
+          const backupCrdExist = await kube.getCrd(this.cheClusterBackupCrd)
+          const restoreCrdExist = await kube.getCrd(this.cheClusterRestoreCrd)
+          if (backupCrdExist && restoreCrdExist) {
+            task.title = `${task.title}...already exist.`
+            return
+          }
+
+          let done = false
+          const [backupCrdFileName, restoreCrdFileName] = await this.getBackupRestoreCrdFilesNames(kube)
+          const backupCrdPath = path.join(ctx.resourcesPath, 'crds', backupCrdFileName)
+          if (!backupCrdExist && fs.existsSync(backupCrdPath)) {
+            await kube.createCrdFromFile(backupCrdPath)
+            done = true
+          }
+
+          const restoreCrdPath = path.join(ctx.resourcesPath, 'crds', restoreCrdFileName)
+          if (!restoreCrdExist && fs.existsSync(restoreCrdPath)) {
+            await kube.createCrdFromFile(restoreCrdPath)
+            done = true
+          }
+
+          if (done) {
+            task.title = `${task.title}...done.`
+          } else {
+            task.title = `${task.title}...skipped.`
+          }
+        },
       },
       {
         title: 'Waiting 5 seconds for the new Kubernetes resources to get flushed',
         task: async (_ctx: any, task: any) => {
           await cli.wait(5000)
           task.title = `${task.title}...done.`
-        }
+        },
       },
       {
         title: `Create deployment ${OPERATOR_DEPLOYMENT_NAME} in namespace ${flags.chenamespace}`,
@@ -201,14 +235,16 @@ export class OperatorTasks {
           if (exist) {
             task.title = `${task.title}...It already exists.`
           } else {
-            await kube.createDeploymentFromFile(path.join(ctx.resourcesPath, 'operator.yaml'), flags.chenamespace, flags['che-operator-image'])
+            const deploymentPath = path.join(ctx.resourcesPath, 'operator.yaml')
+            const operatorDeployment = await this.readOperatorDeployment(deploymentPath, flags)
+            await kube.createDeploymentFrom(operatorDeployment)
             task.title = `${task.title}...done.`
           }
-        }
+        },
       },
       {
         title: 'Operator pod bootstrap',
-        task: () => kubeTasks.podStartTasks(CHE_OPERATOR_SELECTOR, flags.chenamespace)
+        task: () => kubeTasks.podStartTasks(CHE_OPERATOR_SELECTOR, flags.chenamespace),
       },
       {
         title: 'Prepare CodeReady Workspaces cluster CR',
@@ -225,9 +261,9 @@ export class OperatorTasks {
           }
 
           task.title = `${task.title}...Done.`
-        }
+        },
       },
-      createEclipseCheCluster(flags, kube)
+      createEclipseCheCluster(flags, kube),
     ], { renderer: flags['listr-renderer'] as any })
   }
 
@@ -243,16 +279,15 @@ export class OperatorTasks {
           }
           ctx.deployedCheOperatorYaml = operatorDeployment
           task.title = `${task.title}...done`
-        }
+        },
       },
       {
         title: 'Detecting existing version...',
         task: async (ctx: any, task: any) => {
           ctx.deployedCheOperatorImage = this.retrieveContainerImage(ctx.deployedCheOperatorYaml)
-          const deployedCheOperatorImageAndTag = ctx.deployedCheOperatorImage.split(':', 2)
-          ctx.deployedCheOperatorImageName = deployedCheOperatorImageAndTag[0]
-          ctx.deployedCheOperatorImageTag = deployedCheOperatorImageAndTag.length === 2 ? deployedCheOperatorImageAndTag[1] : 'latest'
-          ctx.deployedCheOperatorImage = ctx.deployedCheOperatorImageName + ':' + ctx.deployedCheOperatorImageTag
+          const [deployedImage, deployedTag] = getImageNameAndTag(ctx.deployedCheOperatorImage)
+          ctx.deployedCheOperatorImageName = deployedImage
+          ctx.deployedCheOperatorImageTag = deployedTag
 
           if (flags['che-operator-image']) {
             ctx.newCheOperatorImage = flags['che-operator-image']
@@ -261,14 +296,14 @@ export class OperatorTasks {
             const newCheOperatorYaml = safeLoadFromYamlFile(path.join(flags.templates, OPERATOR_TEMPLATE_DIR, 'operator.yaml')) as V1Deployment
             ctx.newCheOperatorImage = this.retrieveContainerImage(newCheOperatorYaml)
           }
-          const newCheOperatorImageAndTag = ctx.newCheOperatorImage.split(':', 2)
-          ctx.newCheOperatorImageName = newCheOperatorImageAndTag[0]
-          ctx.newCheOperatorImageTag = newCheOperatorImageAndTag.length === 2 ? newCheOperatorImageAndTag[1] : 'latest'
-          ctx.newCheOperatorImage = ctx.newCheOperatorImageName + ':' + ctx.newCheOperatorImageTag
+          const [newImage, newTag] = getImageNameAndTag(ctx.newCheOperatorImage)
+          ctx.newCheOperatorImageName = newImage
+          ctx.newCheOperatorImageTag = newTag
 
           task.title = `${task.title} ${ctx.deployedCheOperatorImageTag} -> ${ctx.newCheOperatorImageTag}`
-        }
-      }])
+        },
+      },
+    ])
   }
 
   updateTasks(flags: any, command: Command): Listr {
@@ -288,55 +323,92 @@ export class OperatorTasks {
             await kube.createServiceAccountFromFile(yamlFilePath, flags.chenamespace)
             task.title = `${task.title}...created new one.`
           }
-        }
+        },
       },
       this.getReadRolesAndBindingsTask(kube),
       this.getCreateOrUpdateRolesAndBindingsTask(flags, 'Updating Roles and Bindings', true),
       {
         title: `Updating CodeReady Workspaces cluster CRD ${this.cheClusterCrd}`,
         task: async (ctx: any, task: any) => {
-          const crd = await kube.getCrd(this.cheClusterCrd)
-          const yamlFilePath = path.join(ctx.resourcesPath, 'crds', 'org_v1_che_crd.yaml')
-          if (crd) {
-            if (!crd.metadata || !crd.metadata.resourceVersion) {
+          const existedCRD = await kube.getCrd(this.cheClusterCrd)
+          const newCRDPath = await this.getCRDPath(ctx, flags)
+
+          if (existedCRD) {
+            if (!existedCRD.metadata || !existedCRD.metadata.resourceVersion) {
               throw new Error(`Fetched CRD ${this.cheClusterCrd} without resource version`)
             }
 
-            await kube.replaceCrdFromFile(yamlFilePath, crd.metadata.resourceVersion)
+            await kube.replaceCrdFromFile(newCRDPath, existedCRD.metadata.resourceVersion)
             task.title = `${task.title}...updated.`
           } else {
-            await kube.createCrdFromFile(yamlFilePath)
+            await kube.createCrdFromFile(newCRDPath)
             task.title = `${task.title}...created new one.`
           }
-        }
+        },
+      },
+      {
+        title: 'Updating backup and restore CRDs',
+        task: async (ctx: any, task: any) => {
+          const [backupCrdFileName, restoreCrdFileName] = await this.getBackupRestoreCrdFilesNames(kube)
+          const existedBackupCRD = await kube.getCrd(this.cheClusterBackupCrd)
+          const newBackupCRDPath = path.join(ctx.resourcesPath, 'crds', backupCrdFileName)
+          if (fs.existsSync(newBackupCRDPath)) {
+            if (existedBackupCRD) {
+              if (!existedBackupCRD.metadata || !existedBackupCRD.metadata.resourceVersion) {
+                throw new Error(`Fetched CRD ${this.cheClusterBackupCrd} without resource version`)
+              }
+              await kube.replaceCrdFromFile(newBackupCRDPath, existedBackupCRD.metadata.resourceVersion)
+            } else {
+              await kube.createCrdFromFile(newBackupCRDPath)
+            }
+          }
+
+          const existedRestoreCRD = await kube.getCrd(this.cheClusterRestoreCrd)
+          const newRestoreCRDPath = path.join(ctx.resourcesPath, 'crds', restoreCrdFileName)
+          if (fs.existsSync(newRestoreCRDPath)) {
+            if (existedRestoreCRD) {
+              if (!existedRestoreCRD.metadata || !existedRestoreCRD.metadata.resourceVersion) {
+                throw new Error(`Fetched CRD ${this.cheClusterRestoreCrd} without resource version`)
+              }
+              await kube.replaceCrdFromFile(newRestoreCRDPath, existedRestoreCRD.metadata.resourceVersion)
+              task.title = `${task.title}...updated.`
+            } else {
+              await kube.createCrdFromFile(newRestoreCRDPath)
+              task.title = `${task.title}...created new one.`
+            }
+          } else {
+            task.title = `${task.title}...skipped.`
+          }
+        },
       },
       {
         title: 'Waiting 5 seconds for the new Kubernetes resources to get flushed',
         task: async (_ctx: any, task: any) => {
           await cli.wait(5000)
           task.title = `${task.title}...done.`
-        }
+        },
       },
       {
         title: `Updating deployment ${OPERATOR_DEPLOYMENT_NAME} in namespace ${flags.chenamespace}`,
         task: async (ctx: any, task: any) => {
           const exist = await kube.deploymentExist(OPERATOR_DEPLOYMENT_NAME, flags.chenamespace)
           const deploymentPath = path.join(ctx.resourcesPath, 'operator.yaml')
+          const operatorDeployment = await this.readOperatorDeployment(deploymentPath, flags)
           if (exist) {
-            await kube.replaceDeploymentFromFile(deploymentPath, flags.chenamespace, flags['che-operator-image'])
+            await kube.replaceDeploymentFrom(operatorDeployment)
             task.title = `${task.title}...updated.`
           } else {
-            await kube.createDeploymentFromFile(deploymentPath, flags.chenamespace, flags['che-operator-image'])
+            await kube.createDeploymentFrom(operatorDeployment)
             task.title = `${task.title}...created new one.`
           }
-        }
+        },
       },
       {
         title: 'Waiting newer operator to be run',
         task: async (_ctx: any, _task: any) => {
           await cli.wait(1000)
           await kube.waitLatestReplica(OPERATOR_DEPLOYMENT_NAME, flags.chenamespace)
-        }
+        },
       },
       patchingEclipseCheCluster(flags, kube, command),
     ], { renderer: flags['listr-renderer'] as any })
@@ -346,7 +418,7 @@ export class OperatorTasks {
    * Returns list of tasks which remove CodeReady Workspaces operator related resources
    */
   deleteTasks(flags: any): ReadonlyArray<Listr.ListrTask> {
-    let kh = new KubeHelper(flags)
+    const kh = new KubeHelper(flags)
     return [{
       title: 'Delete oauthClientAuthorizations',
       task: async (_ctx: any, task: any) => {
@@ -356,50 +428,71 @@ export class OperatorTasks {
           await kh.deleteOAuthClientAuthorizations(oAuthClientAuthorizations)
         }
         task.title = `${task.title}...OK`
-      }
+      },
     },
     {
       title: `Delete the Custom Resource of type ${CHE_CLUSTER_CRD}`,
       task: async (_ctx: any, task: any) => {
-        const checluster = await kh.getCheCluster(flags.chenamespace)
-        if (checluster) {
-          await kh.patchCheClusterCustomResource(checluster.metadata.name, flags.chenamespace, { metadata: { finalizers: null } })
+        await kh.deleteCheCluster(flags.chenamespace)
+
+        // wait 20 seconds, default timeout in che operator
+        for (let index = 0; index < 20; index++) {
+          await cli.wait(1000)
+          if (!await kh.getCheCluster(flags.chenamespace)) {
+            task.title = `${task.title}...OK`
+            return
+          }
         }
 
-        await kh.deleteCheCluster(flags.chenamespace)
-        do {
-          await cli.wait(2000) //wait a couple of secs for the finalizers to be executed
-        } while (await kh.getCheCluster(flags.chenamespace))
-        task.title = `${task.title}...OK`
-      }
+        // if checluster still exists then remove finalizers and delete again
+        const checluster = await kh.getCheCluster(flags.chenamespace)
+        if (checluster) {
+          try {
+            await kh.patchCustomResource(checluster.metadata.name, flags.chenamespace, { metadata: { finalizers: null } }, CHE_CLUSTER_API_GROUP, CHE_CLUSTER_API_VERSION, CHE_CLUSTER_KIND_PLURAL)
+          } catch (error) {
+            if (await kh.getCheCluster(flags.chenamespace)) {
+              task.title = `${task.title}...OK`
+              return // successfully removed
+            }
+            throw error
+          }
+
+          // wait 2 seconds
+          await cli.wait(2000)
+        }
+
+        if (!await kh.getCheCluster(flags.chenamespace)) {
+          task.title = `${task.title}...OK`
+        } else {
+          task.title = `${task.title}...Failed`
+        }
+      },
     },
     {
-      title: `Delete CRD ${this.cheClusterCrd}`,
+      title: 'Delete CRDs',
       task: async (_ctx: any, task: any) => {
-        const crdExists = await kh.crdExist(this.cheClusterCrd)
         const checlusters = await kh.getAllCheClusters()
         if (checlusters.length > 0) {
           task.title = `${task.title}...Skipped: another CodeReady Workspaces deployment found.`
         } else {
-          // Check if CRD exist. When installer is helm the CRD are not created
-          if (crdExists) {
-            await kh.deleteCrd(this.cheClusterCrd)
-          }
+          await kh.deleteCrd(this.cheClusterCrd)
+          await kh.deleteCrd(this.cheClusterBackupCrd)
+          await kh.deleteCrd(this.cheClusterRestoreCrd)
           task.title = `${task.title}...OK`
         }
-      }
+      },
     },
     {
       title: 'Delete Roles and Bindings',
       task: async (_ctx: any, task: any) => {
         const roleBindings = await kh.listRoleBindings(flags.chenamespace)
         for (const roleBinding of roleBindings.items) {
-          await kh.deleteRoleBinding(roleBinding.metadata!.name, flags.chenamespace)
+          await kh.deleteRoleBinding(roleBinding.metadata!.name!, flags.chenamespace)
         }
 
         const roles = await kh.listRoles(flags.chenamespace)
         for (const role of roles.items) {
-          await kh.deleteRole(role.metadata!.name, flags.chenamespace)
+          await kh.deleteRole(role.metadata!.name!, flags.chenamespace)
         }
 
         // Count existing pairs of cluster roles and thier bindings
@@ -424,56 +517,107 @@ export class OperatorTasks {
 
         // If no pairs were deleted, then legacy names is used
         if (pairs === 0) {
-          if (await kh.clusterRoleBindingExist(this.legacyClusterResourcesName)) {
-            await kh.deleteClusterRoleBinding(this.legacyClusterResourcesName)
-          }
-          if (await kh.clusterRoleExist(this.legacyClusterResourcesName)) {
-            await kh.deleteClusterRole(this.legacyClusterResourcesName)
-          }
+          await kh.deleteClusterRoleBinding(this.legacyClusterResourcesName)
+          await kh.deleteClusterRole(this.legacyClusterResourcesName)
         }
 
         task.title = `${task.title}...OK`
-      }
+      },
     },
     {
       title: `Delete service accounts ${this.operatorServiceAccount}`,
       task: async (_ctx: any, task: any) => {
-        if (await kh.serviceAccountExist(this.operatorServiceAccount, flags.chenamespace)) {
-          await kh.deleteServiceAccount(this.operatorServiceAccount, flags.chenamespace)
-        }
+        await kh.deleteServiceAccount(this.operatorServiceAccount, flags.chenamespace)
         task.title = `${task.title}...OK`
-      }
+      },
     },
     {
       title: 'Delete PVC codeready-operator',
       task: async (_ctx: any, task: any) => {
-        if (await kh.persistentVolumeClaimExist('codeready-operator', flags.chenamespace)) {
-          await kh.deletePersistentVolumeClaim('codeready-operator', flags.chenamespace)
-        }
+        await kh.deletePersistentVolumeClaim('codeready-operator', flags.chenamespace)
         task.title = `${task.title}...OK`
-      }
-    },
-    ]
+      },
+    }]
   }
 
   retrieveContainerImage(deployment: V1Deployment) {
     const containers = deployment.spec!.template!.spec!.containers
-
     const namespace = deployment.metadata!.namespace
     const name = deployment.metadata!.name
-    if (containers.length === 0) {
+    const container = containers.find(c => c.name === 'codeready-operator')
+
+    if (!container) {
       throw new Error(`Can not evaluate image of ${namespace}/${name} deployment. Containers list are empty`)
     }
-
-    if (containers.length > 1) {
-      throw new Error(`Can not evaluate image of ${namespace}/${name} deployment. It has multiple containers`)
-    }
-
-    const container = containers[0]
     if (!container.image) {
       throw new Error(`Container ${container.name} in deployment ${namespace}/${name} must have image specified`)
     }
 
     return container.image
   }
+
+  async getCRDPath(ctx: any, flags: any): Promise<string> {
+    let newCRDFilePath: string
+
+    const kube = new KubeHelper(flags)
+    if (!await kube.IsAPIExtensionSupported('v1')) {
+      // try to get CRD v1beta1 if platform doesn't support v1
+      newCRDFilePath = path.join(ctx.resourcesPath, 'crds', 'org_v1_che_crd-v1beta1.yaml')
+      if (fs.existsSync(newCRDFilePath)) {
+        return newCRDFilePath
+      }
+    }
+
+    return path.join(ctx.resourcesPath, 'crds', 'org_v1_che_crd.yaml')
+  }
+
+  // Delete this method and use default v1 CRDs when Openshift 3.x support dropped
+  private async getBackupRestoreCrdFilesNames(kube: KubeHelper): Promise<[string, string]> {
+    let backupCrdFileName: string
+    let restoreCrdFileName: string
+    if (!await kube.IsAPIExtensionSupported('v1')) {
+      // Needed for Openshift 3.x
+      backupCrdFileName = 'org.eclipse.che_checlusterbackups_crd-v1beta1.yaml'
+      restoreCrdFileName = 'org.eclipse.che_checlusterrestores_crd-v1beta1.yaml'
+    } else {
+      backupCrdFileName = 'org.eclipse.che_checlusterbackups_crd.yaml'
+      restoreCrdFileName = 'org.eclipse.che_checlusterrestores_crd.yaml'
+    }
+    return [backupCrdFileName, restoreCrdFileName]
+  }
+
+  /**
+   * Reads and patch 'codeready-operator' deployment:
+   * - sets operator image
+   * - sets deployment namespace
+   * - removes other containers for ocp 3.11
+   */
+  private async readOperatorDeployment(path: string, flags: any): Promise<V1Deployment> {
+    const operatorDeployment = safeLoadFromYamlFile(path) as V1Deployment
+
+    if (!operatorDeployment.metadata || !operatorDeployment.metadata!.name) {
+      throw new Error(`Deployment read from ${path} must have name specified`)
+    }
+
+    if (flags['che-operator-image']) {
+      const container = operatorDeployment.spec!.template.spec!.containers.find(c => c.name === 'codeready-operator')
+      if (container) {
+        container.image = flags['che-operator-image']
+      } else {
+        throw new Error(`Container 'codeready-operator' not found in deployment '${operatorDeployment.metadata!.name}'`)
+      }
+    }
+
+    if (flags.chenamespace) {
+      operatorDeployment.metadata!.namespace = flags.chenamespace
+    }
+
+    const kube = new KubeHelper(flags)
+    if (!await kube.IsAPIExtensionSupported('v1')) {
+      const containers = operatorDeployment.spec!.template.spec!.containers || []
+      operatorDeployment.spec!.template.spec!.containers = containers.filter(c => c.name === 'codeready-operator')
+    }
+
+    return operatorDeployment
+  }
 }
